﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Linq;

namespace Takistusrada
{
    class Program
    {
        private const int N = 10;
        private const int MaxSpeed = 7;
        private const int MaxCoord = (MaxSpeed*(MaxSpeed + 1))/2 + N;
        private const int CSpace = 2*MaxCoord + 1;
        private const int SSpace = 2*MaxSpeed + 1;
        private const int MaxSpace = CSpace*CSpace*SSpace*SSpace;

        static void Main(string[] args)
        {
            if (args.Length > 0 && args[0] == "all")
            {
                for (int i = 0; i <= 25; i++)
                {
                    string input = "input" + i + ".txt";
                    if (File.Exists(input))
                    {
                        Solve(input, "output" + i + ".txt");
                    }
                }
            }
            else
            {
                Solve("takipsis.txt", "takipval.txt");
            }
        }

        static bool[] _visited = new bool[MaxSpace];
        static Queue<Tuple<Point, Point, int>> _q;  // loc, speed, order
        private static List<Tuple<Point, Point>> _obstacles;
        private static void Solve(string sis, string val)
        {
            DateTime start = DateTime.Now;
            _visited = new bool[MaxSpace];
            string[] lines = File.ReadAllLines(sis);
            int res = -1;

            int[] firstLine = lines[0].Split().Select(int.Parse).ToArray();
            Point target = new Point(firstLine[0], firstLine[1]);

            _obstacles = lines.Skip(1).Select(
                line => line.Split().Select(int.Parse).ToArray()).Select(
                    coords => coords.Length == 2 ?
                        new Tuple<Point, Point>(new Point(coords[0], coords[1]), new Point(coords[0], coords[1])) :
                        new Tuple<Point, Point>(new Point(coords[0], coords[1]), new Point(coords[2], coords[3]))).ToList();

            _q = new Queue<Tuple<Point, Point, int>>();  // loc, speed, order
            _q.Enqueue(new Tuple<Point, Point, int>(new Point(0, 0), new Point(0, 0), 0));

            long tries = 0;
            while (_q.Count > 0)
            {
                tries++;
                Tuple<Point, Point, int> state = _q.Dequeue();
                if (state.Item1 == target)
                {
                    res = state.Item3;
                    break;
                }

                TryDirection(1, 0, state);
                TryDirection(-1, 0, state);
                TryDirection(0, 0, state);
                TryDirection(0, 1, state);
                TryDirection(0, -1, state);

                if (_q.Count > MaxSpace) 
                    break;
            }

            File.WriteAllText(val, res.ToString());
            TimeSpan elapsed = DateTime.Now - start;
            Console.WriteLine(sis + " " + elapsed);
        }

        private static int maxSpeed = 0;
        private static int maxCoord = 0;
        private static void TryDirection(int dx, int dy, Tuple<Point, Point, int> state)
        {
            Point speed = new Point(state.Item2.X + dx, state.Item2.Y + dy);
            Point endpoint = new Point(state.Item1.X + speed.X, state.Item1.Y + speed.Y);
            if (Math.Abs(speed.X) > MaxSpeed || Math.Abs(speed.Y) > MaxSpeed)
            {
                return;
            }
            if (Math.Abs(endpoint.X) > MaxCoord || Math.Abs(endpoint.Y) > MaxCoord)
            {
                return;
            }

            int key = (MaxCoord + endpoint.X) * CSpace * SSpace * SSpace + (endpoint.Y + MaxCoord) * SSpace * SSpace + (speed.X + MaxSpeed) * SSpace + (speed.Y + MaxSpeed);
            if (_visited[key]) return;
            _visited[key] = true;

            //maxSpeed = Math.Max(maxSpeed, speed.X);
            //maxSpeed = Math.Max(maxSpeed, speed.kY);
            //maxCoord = Math.Max(maxCoord, endpoint.X);
            //maxCoord = Math.Max(maxCoord, endpoint.Y);
            if (_obstacles.Any(obstacle => LinesIntersect(state.Item1, endpoint, obstacle.Item1, obstacle.Item2)))
            {
                return;
            }

            _q.Enqueue(new Tuple<Point, Point, int>(endpoint, speed, state.Item3 + 1));
        }

        static bool LinesIntersect(Point p11, Point p12, Point p21, Point p22)
        {
            Point cmP = new Point(p21.X - p11.X, p21.Y - p11.Y);
            Point r = new Point(p12.X - p11.X, p12.Y - p11.Y);
            Point s = new Point(p22.X - p21.X, p22.Y - p21.Y);

            int cmPxr = cmP.X * r.Y - cmP.Y * r.X;
            int cmPxs = cmP.X * s.Y - cmP.Y * s.X;
            int rxs = r.X * s.Y - r.Y * s.X;

            if (cmPxr == 0)
            {
                // Lines are collinear, and so intersect if they have any overlap
                return 
                    ((p21.X - p11.X < 0f) != (p21.X - p12.X < 0f)) || 
                    ((p21.Y - p11.Y < 0f) != (p21.Y - p12.Y < 0f)) ||
                    p21 == p11 || p21 == p12 || p22 == p11 || p22 == p12;
            }

            if (rxs == 0)
                return false; // Lines are parallel.

            float rxsr = 1f / rxs;
            float t = cmPxs * rxsr;
            float u = cmPxr * rxsr;

            return (t >= 0f) && (t <= 1f) && (u >= 0f) && (u <= 1f);
        }
    }
}
